/**
* Filename: Serializable.java (in org.openbandy.io)
* This is part of the OpenBandy project.

* @author Philipp Bolliger (philipp@bolliger.name)
* @version 1.3
* 
* (c) Copyright 2007, ALL RIGHTS RESERVED.
*
* Classes that need to be stored in the database or the Record Management
* Store, or that need to be transformed into a stream can implement
* this interface. Classes of this type can thereafter be serialized using
* the appropriate serializer.
*/

package org.openbandy.io;

import java.util.Vector;


public interface Serializable {
    
    /**
	 * This method must fill the container provided with the serialization-
	 * Object with values for every attribute denoted in getAttributeNames().
	 * Subsequently it must serialize member objects as denoted in getReferences().
     * 
     * @param serializer - the serializer used for callbacks
     * @param serializationBuffer - the serializationBuffer containing the container to write values to
     */
    public void serialize(Serializer serializer, SerializationBuffer serializationBuffer);
    
    /**
     * This method must overwrite the attributes denoted in getAttributeNames()
     * with values read from the container provided. Subsequently it must de-
     * serialize member objects as denoted in getReferences().
     * 
     * @param serializer - the serializer used for callbacks
     * @param container - the container to read values from
     */
    public void deserialize(Serializer serializer, SerializationBuffer serializationBuffer);
    
    /**
     * Must return the name of the id (e.g. eventId).
     * 
     * @return the name of the id
     */
    public String getIdName();
    
    /**
     * Must return the value of the id.
     * 
     * @return the actual value of id
     */
    public int getIdValue();
    
    /**
     * Set the id to the given value.
     * 
     * @param idValue -  the new id value
     */
    public void setIdValue(int idValue);
    
    /**
     * This method must put the name of every attribute that
     * needs to be serialized in the given Vector. For every
     * String value contained in <code>attributes</code>, there
     * must be a corresponding field in the database for example.
     * 
     * @param attributes - the vector to be filled
     */
    public void getAttributeNames(Vector attributes);
    
    /**
     * This method must put every reference to Serializable
     * objects, i.e. member objects, in the given Vector.
     * 
     * @param references - the vector to be filled
     */
    public void getReferences(Vector references);
    
    /**
     * This method must put every id name of referenced
     * objects, i.e. member objects that need to be
     * serialized, in the given Vector.
     * 
     * @param referencedIdNames - the vector to be filled
     */
    public void getReferencedIdNames(Vector referencedIdNames);
    
    
    /**
     * Must return a human readable String representation
     * of the object.
     * 
     * @return human legible string
     */
    public String getLegibleString();
    
    
    /**
     * Must return true if the serializableObject is equal to
     * this, except for the id but INCLUDING equality of 
     * referenced serializables.
     * 
     * @param  serializableObject - the object to compare
     * @return true if serializableObject is equal to this
     * @throws ClassCastException
     */
    public boolean isEqual(Serializable serializableObject) throws ClassCastException;
    
    /**
     * Must return true if the serializableObject is equal to
     * this, except for the id, EXCLUDING equality of 
     * referenced serializables.
     * 
     * @param  serializableObject - the object to compare
     * @return true if serializableObject is equal to this
     * @throws ClassCastException
     */
    public boolean isEqualShallow(Serializable serializableObject) throws ClassCastException;
    
    /**
     * Must return a new and initialized instance.
     * 
     * @return New instance of concrete class.
     */
    public Serializable getNewInstance();
    
    /**
     * Must return a copy of the object that is equal according
     * to the method isEuqalTo().
     * 
     * @return copy of this
     */
    public Serializable getCopy();
}
