/**
* Filename: SerializationIdentifiers.java (in org.openbandy.io)
* This is part of the OpenBandy project.

* @author Philipp Bolliger (philipp@bolliger.name)
* @version 0.7
* 
* (c) Copyright 2007, ALL RIGHTS RESERVED.
*
* Every storage serializable object has a numeric id. Thereby applies the
* following categorization:
* 1. Objects that have never been serialized have an ID value of category NO ID
* 2. Objects that have only been serialized to the RMS have an ID value of category RMS ID
* 3. Objects that have at least once been serialized to the database have an ID value of
*    category DB ID
*/

package org.openbandy.io;

import java.util.Enumeration;
import java.util.Vector;

public abstract class SerializationIdentifiers {
	
	public final static int NO_ID = -1;
	public final static int RMS_ID_MIN = 2<<29;
	public final static int RMS_ID_MAX = Integer.MAX_VALUE;
	
	/**
	 * Checks whether <code>id</code> is valid. 
	 * @param id The id to check
	 * @return True if <code>id</code> is valid
	 */
	public static boolean isValidId(int id){
		return (id>0);
	}
	
	/**
	 * Checks whether <code>id</code> is a valid RMS ID. 
	 * @param id The id to check for RMS validity
	 * @return True if <code>id</code> is valid
	 */
	public static boolean isRmsSerializableId(int id){
		return (id >= RMS_ID_MIN);
	}
	
	/**
	 * Checks whether <code>id</code> is a valid database ID. 
	 * @param id The id to check for database validity
	 * @return True if <code>id</code> is valid
	 */
	public static boolean isDatabaseSerializableId(int id){
		return ( (isValidId(id)) && (!isRmsSerializableId(id)) );
	}
	
	/**
	 * TODO comment!
	 *
	 * @param serializable
	 */
	public static void resetAllIds(Serializable serializable){
		serializable.setIdValue(NO_ID);
		
		/* reset referenced objects' ids */
		Vector references = new Vector();
		serializable.getReferences(references);
		Enumeration elements = references.elements();
		while(elements.hasMoreElements()){
			Serializable referenceedSerializable = (Serializable)elements.nextElement();
			if(referenceedSerializable != null){
				resetAllIds(referenceedSerializable);
			}
		}
	}
	
}
