package org.openbandy.pref;

import java.util.Enumeration;
import java.util.Hashtable;

import javax.microedition.rms.InvalidRecordIDException;
import javax.microedition.rms.RecordStore;
import javax.microedition.rms.RecordStoreException;
import javax.microedition.rms.RecordStoreFullException;
import javax.microedition.rms.RecordStoreNotOpenException;

/**
 * TODO describe purpose and usage
 *
 * Used to store name, value pairs in the RMS. Uses one store
 * named 'Preferences'.
 * 
 * whole class is forbidden to use the logger as the log itself uses prefs!
 * 
 * @version 1.0
 * @author Philipp Bolliger (philipp@bolliger.name)
 */
public final class Preferences {

	/** The name of the record store that is used to store the preferences names */
	private static final String PREFERENCES_RECORD_STORE_NAME = "Preferences";

	/** Character used to delimit the name, the value and the boolean indicator
	 *  <code>screenModifiable</code> when storing one string to the
	 *  PREF_NAMES_RECORD_STORE_NAME record store */
	protected static final char DELIMITER = ':';

	/** Record store used to save the name, the value and the boolean indicator
	 * <code>screenModifiable</code> of a prefernce, delimited by <code>ID_DELIMITER</code>. */
	private static RecordStore preferencesRecordStore;

	/** The preferences:  name (String) -> preference (Preference) */
	private static Hashtable preferences = new Hashtable();

	/* init */
	static{
		try {
			/* open the record store */
			preferencesRecordStore = RecordStore.openRecordStore(PREFERENCES_RECORD_STORE_NAME, true);

			/* load existing preferences from the RMS */
			for (int id = 1; id < preferencesRecordStore.getNextRecordID(); id++) {
				try{
					/* read the record store entry into a string */
					byte[] record = preferencesRecordStore.getRecord(id);
					String recordStoreEntry = new String(record);
					
					/* create new preference and put it into hashtable */
					Preference preference = new Preference(id, recordStoreEntry);
					preferences.put(preference.name, preference);
				}
				catch (InvalidRecordIDException irie) {
					/* ignore */
				}
			}
		}
		catch (Exception e) {
			/* ignore */
		}
	}

	/**
	 * @param name
	 * @param screenModifiable if true, the preference will appear in the PreferencesForm screen (@see PreferencesForm)
	 * @throws Exception
	 * 
	 */
	public static void add(String name, boolean screenModifiable) throws Exception {
		/* check if not already existing */
		if(! preferences.containsKey(name)) {
			/* create the preference */
			Preference preference = new Preference(name, "", screenModifiable);

			/* add the preference to record store */
			byte[] entry = preference.getBytes();
			preference.id = preferencesRecordStore.addRecord(entry,0,entry.length);
			
			/* put newly added preference and id in hashtables */
			preferences.put(name, preference);
		}
	}

	/**
	 * @param name
	 * @throws UnknownPreferenceException
	 */
	public static void remove(String name) throws UnknownPreferenceException, RecordStoreException, InvalidRecordIDException, RecordStoreNotOpenException {
		if(preferences.containsKey(name)){
			/* get preference from hashtable */
			Preference preference = (Preference)preferences.get(name);
			
			/* delete the record */
			preferencesRecordStore.deleteRecord(preference.id);

			/* remove the preference from the hashtable */
			preferences.remove(name);
		}
		else {
			throw new UnknownPreferenceException("Can not remove '"+name+"', unknown preference.");
		}
	}

	/**
	 * @param name
	 * @return
	 * @throws UnknownPreferenceException
	 */
	public static String getValue(String name) throws UnknownPreferenceException {
		if(preferences.containsKey(name)) {
			/* get preference from hashtable and return its value */
			Preference preference = (Preference)preferences.get(name);
			return preference.value;
		}
		else {
			throw new UnknownPreferenceException("Can not get value for '"+name+"', unknown preference.");
		}
	}

	/**
	 * @param name
	 * @return
	 * @throws UnknownPreferenceException
	 */
	public static int getIntValue(String name) throws InvalidValueException, UnknownPreferenceException {
		String value = getValue(name);
		try {
			return Integer.parseInt(value);
		} catch (NumberFormatException  e) {
			throw new InvalidValueException("Value is not an integer: "+value);
		}
	}

	/**
	 * @param name
	 * @return
	 * @throws UnknownPreferenceException
	 */
	public static boolean getBooleanValue(String name) throws InvalidValueException, UnknownPreferenceException {
		String value = getValue(name);
		if(value.equals(String.valueOf(true))){
			return true;
		}
		else if(value.equals(String.valueOf(false))){
			return false;
		}
		else {
			throw new InvalidValueException("Value is not a boolean: "+value);
		}
	}

	/**
	 * if the preference does not already exist, <code>add()</code> it
	 * however, newly added preferences are not screen modifiable
	 * 
	 * @param name
	 * @param value
	 */
	public static void setValue(String name, String value) throws InvalidValueException, RecordStoreFullException, RecordStoreNotOpenException, RecordStoreException {
		/* ensure that the value does not contain the DELIMITER char */
		if(value.indexOf(DELIMITER) != -1){
			throw new InvalidValueException("Value must not contain '"+DELIMITER+"'");
		}
		
		/* if the preference does not already exists, add it*/
		if(!preferences.containsKey(name)) {
			/* create the preference */
			Preference preference = new Preference(name, value, false);
			byte[] entry = preference.getBytes();

			/* add the preference to record store */
			preference.id = preferencesRecordStore.addRecord(entry,0,entry.length);
			
			/* put newly added preference and id in hashtables */
			preferences.put(name, preference);
		}
		else {
			/* get preference from hashtable */
			Preference preference = (Preference)preferences.get(name);
			
			/* set new value */
			preference.value = value;
			
			/* update record */
			byte[] entry = preference.getBytes();
			preferencesRecordStore.setRecord(preference.id, entry, 0, entry.length);
		}
	}

	/**
	 * @param name
	 * @param value
	 * @throws UnknownPreferenceException
	 */
	public static void setIntValue(String name, int value) throws InvalidValueException, RecordStoreFullException, RecordStoreNotOpenException, RecordStoreException  {
		setValue(name, Integer.toString(value));
	}

	/**
	 * @param name
	 * @param value
	 * @throws UnknownPreferenceException
	 */
	public static void setBooleanValue(String name, boolean value) throws InvalidValueException, RecordStoreFullException, RecordStoreNotOpenException, RecordStoreException  {
		setValue(name, String.valueOf(value));
	}
	
	/**
	 * @return
	 */
	public static Enumeration preferences() {
		return preferences.elements();
	}

}
