/**
* Filename: EventRouterImpl.java (in org.openbandy.event)
* This file is part of the BandyBasics project.
*
* www.openbandy.org
*/

package org.openbandy.service;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import org.openbandy.event.Event;
import org.openbandy.event.EventDispatcher;
import org.openbandy.event.EventFilter;
import org.openbandy.event.EventSink;
import org.openbandy.event.EventSource;
import org.openbandy.event.EventSourceBroker;
import org.openbandy.event.EventSourceController;
import org.openbandy.util.StringUtil;

/**
 * TODO describe purpose and usage
 * 
 * DOKU auflsung source->sink ber short class name des events !!!
 * 
 * As soon as an EventSink is registered @ the EventRouter,
 * the router tries to start the needed eventsources. If no sources
 * are available, the eventsink stays registered and corresponding
 * sources are started as soon as they are registered at the eventrouter.
 * If an eventSource is registered that provides an eventType that is
 * absorbed by a registered eventSink, the affected eventSink will be
 * informed via <code>lostEventSource</code>. 
 *
 * 
 * <br><br>
 * (c) Copyright Philipp Bolliger 2007, ALL RIGHTS RESERVED.
 * 
 * @author Philipp Bolliger (philipp@bolliger.name)
 * @version 0.9
 */
public final class EventService extends Service {
    
    /* A map of dispatchers for known Events */
    private static Hashtable eventDispatchers = new Hashtable();
    
    /* A map of eventSourceControllers for known event types */
    private static Hashtable eventSourceControllers = new Hashtable();
    
    /* A pool of registered event sources */
    private static EventSourceBroker eventSourceBroker = new EventSourceBroker();

	/* Hide constructor */
	private EventService(){}
    
    /* *************** Methods for EventRouter ***************** */
    
    /**
     * This method registers an EventSource with every type of
     * event it provides.
     * A thus registered source will be started as soon as a
     * sink registers for a corresponding event type. If no
     * sink is interested in the type of event provided by this
     * source, it will be stopped eventually. 
     *
     * @param eventSource The source to register.
     */
    public static synchronized void registerSource(EventSource eventSource) {
        
        /* register the event source */
        Integer eventSourceIndex = eventSourceBroker.addEventSource(eventSource);
        
        if(eventSourceIndex.intValue() > 0){
            
            /* make an event source controller entry for every provided event type */
            Vector providedEventTypes = eventSource.getProvidedEventTypes();
            if(providedEventTypes != null){
                
                Enumeration elements = providedEventTypes.elements();
                while(elements.hasMoreElements()){
                    try{
                        String providedEventType = (String)elements.nextElement();
                        
                        EventSourceController sourceController = getSourceController(providedEventType);
                        sourceController.addSource(eventSourceIndex);
                        LogService.debug("EventRouter", "Registered EventSource "+eventSource.toString()+" providing "+providedEventType);
                        
                        /* if there are sinks subscribed for this type of event, ensure running */
                        if(hasRegisteredSink(providedEventType)){
                            sourceController.ensureRunning();
                        }
                    }catch(ClassCastException cce){
                    	LogService.error("EventRouter", "Event Type not given as String", cce);
                    }
                }
            }
        }
        
    }
    
    
    /**
     * This method registers an EventSink with every type of event
     * it absorbs. Every event that is absorbed by this sink will
     * henceforth be delegate to this sink.
     * If an EventFilter is provided, the events will be delegated
     * to the filter which then eventually forwards the events to 
     * the subscribed EventSink.
     * 
     * @param eventSink The sink to register.
     */
    public static synchronized void subscribe(EventSink eventSink, EventFilter filter) {
        /* make an event dispatcher entry for every provided event type */
        Vector absorbedEventTypes = eventSink.getAbsorbedEventTypes();
        if(absorbedEventTypes != null){
            
            Enumeration elements = absorbedEventTypes.elements();
            while(elements.hasMoreElements()){
                try{
                    String absorbedEventType = (String)elements.nextElement();
                    
                    /* create an event dispatcher and add sink along with filter */
                    EventDispatcher dispatcher = getDispatcher(absorbedEventType);
                    dispatcher.addSink(eventSink, filter);
                    if( (filter != null) && (filter.filtersEventType(absorbedEventType))){
                    	LogService.debug("EventRouter", "Registered EventSink "+eventSink.toString()+" absorbing "+absorbedEventType+" with filter");
                    }else{
                    	LogService.debug("EventRouter", "Registered EventSink "+eventSink.toString()+" absorbing "+absorbedEventType);
                    }
                    
                    /* check that needed sources are running */
                    if(hasRegisteredSource(absorbedEventType)){
                        EventSourceController sourceController = getSourceController(absorbedEventType);
                        sourceController.ensureRunning();
                    }else{
                    	LogService.debug("EventRouter", "No source registered that would provide events of type :"+absorbedEventType);
                    }
                    
                }catch(ClassCastException cce){
                	LogService.warn("EventRouter", "Event Type not given as String");
                }
            }
        }
    }
    
    /**
     * Unregister an EventSource.
     * 
     * @param eventSource The source to unregister.
     */
    public static synchronized void unregisterSource(EventSource eventSource) {
        /* remove event source from broker */
        if(eventSourceBroker.removeEventSource(eventSource)){
            
            Integer eventSourceIndex = eventSourceBroker.getEventSourceIndex(eventSource);
            
            Enumeration elements = eventSourceControllers.elements();
            while(elements.hasMoreElements()){
                EventSourceController sourceController = (EventSourceController)elements.nextElement();
                if(sourceController.removeSource(eventSourceIndex)){
                	LogService.debug("EventRouter", "Unregistered EventSource "+eventSource.toString());
                    
                    /* if no sources are registered, inform sinks */
                    if(sourceController.isEmpty()){
                        String eventShortClassName = sourceController.getEventShortClassName();
                        
                        EventDispatcher dispatcher = getDispatcher(eventShortClassName);
                        dispatcher.informLostSource();
                    }
                }
            }
        }
    }
    
    
    /**
     * Unsubscribe an EventSink.
     * 
     * @param eventSink The sink to unregister.
     */
    public static synchronized void unsubscribe(EventSink eventSink) {
        Enumeration elements = eventDispatchers.elements();
        while(elements.hasMoreElements()){
            EventDispatcher dispatcher = (EventDispatcher)elements.nextElement();
            if(dispatcher.removeSink(eventSink)){
            	LogService.debug("EventRouter", "Unregistered EventSink "+eventSink.toString());
                
                /* if no sinks are registered, stop sources */
                if(dispatcher.isEmpty()){
                    String eventShortClassName = dispatcher.getEventShortClassName();
                    
                    EventSourceController sourceController = getSourceController(eventShortClassName);
                    sourceController.stopSources();
                }
                
            }
        }
        
    }
    
    
    /**
     * This methods forwards the provided Event to every
     * EventSink that hat been registered for this type of Event.
     * Registration (i.e. distinction) of Events is made by the
     * Events shortClassName (@see StringUtil), as for example
     * 'ObjectEvent'.
     * 
     * @param event The event that has to be routed.
     */
    public static void delegate(Event event) {
        EventDispatcher dispatcher = getDispatcher(event);
        dispatcher.dispatchEvent(event);
    }
    
    
    /* *************** Helper Methods ***************** */
    
    private static EventDispatcher getDispatcher(Event event){
        String eventShortClassName = StringUtil.getShortClassName(event);
        return getDispatcher(eventShortClassName);
    }
    
    private  static EventDispatcher getDispatcher(String eventShortClassName){
        EventDispatcher dispatcher = (EventDispatcher)eventDispatchers.get(eventShortClassName);
        if(dispatcher == null){
            dispatcher = new EventDispatcher(eventShortClassName);
            eventDispatchers.put(eventShortClassName, dispatcher);
        }
        return dispatcher;
    }
    
    private static boolean hasRegisteredSource(String eventShortClassName){
        return eventSourceControllers.containsKey(eventShortClassName);
    }
    
    private static EventSourceController getSourceController(String eventShortClassName){
        EventSourceController sourceController = (EventSourceController)eventSourceControllers.get(eventShortClassName);
        if(sourceController == null){
            sourceController = new EventSourceController(eventShortClassName, eventSourceBroker);
            eventSourceControllers.put(eventShortClassName, sourceController);
        }
        return sourceController;
    }
    
    private static boolean hasRegisteredSink(String eventShortClassName){
        return eventDispatchers.contains(eventShortClassName);
    }
    
    
    //TODO remove dev code
    public static void printRouting(){
        
        System.out.println("");
        System.out.println("EVENT ROUTING");
        
        System.out.println("- Registered Sources");
        Enumeration sources = eventSourceBroker.getSources();
        while(sources.hasMoreElements()){
            EventSource eventSource = (EventSource)sources.nextElement();
            Integer eventSourceIndex = eventSourceBroker.getEventSourceIndex(eventSource);
            System.out.println("-- EventSource: "+eventSource.getName()+" "+eventSource.toString()+" with index: "+eventSourceIndex);
        }
        
        System.out.println("\n- Sources Controller");
        Enumeration controllers = eventSourceControllers.elements();
        while(controllers.hasMoreElements()){
            EventSourceController esc = (EventSourceController)controllers.nextElement();
            System.out.println("\n-- EventSourceController for "+esc.getEventShortClassName());
            
            Hashtable registeredSources = esc.getRegisteredSources();
            Enumeration sourceIndexes = registeredSources.elements();
            while(sourceIndexes.hasMoreElements()){
                Integer esi = (Integer)sourceIndexes.nextElement();
                System.out.println("--- EventSourceIndex: "+esi);
            }
        }
        
        System.out.println("");
        System.out.println("\n- Sink Dispatcher");
        Enumeration dispatchers = eventDispatchers.elements();
        while(dispatchers.hasMoreElements()){
            EventDispatcher ed = (EventDispatcher)dispatchers.nextElement();
            System.out.println("\n-- EventDispatcher for "+ed.getEventShortClassName());
            
            Enumeration sinks = ed.getRegisteredEventSinks().elements();
            while(sinks.hasMoreElements()){
                EventSink es = (EventSink)sinks.nextElement();
                System.out.println("--- EventSink: "+es.toString());
            }
        }
        
    }

}
